/*
 * Elite - The New Kind.
 *
 * Reverse engineered from the BBC disk version of Elite.
 * Additional material by C.J.Pinder.
 *
 * The original Elite code is (C) I.Bell & D.Braben 1984.
 * This version re-engineered in C by C.J.Pinder 1999-2000.
 *
 * email: <christian@newkind.co.uk>
 *
 *
 */

/*
 * alg_main.c
 *
 * Allegro version of the main game handler.
 */


#include <stdio.h>
#include <string.h>
#include <math.h> 
#include <ctype.h>
#include <time.h>

#include "config.h"
#include "gfx.h"
#include "main.h"
#include "vector.h"
#include "alg_data.h"
#include "elite.h"
#include "docked.h"
#include "intro.h"
#include "shipdata.h"
#include "shipface.h"
#include "space.h"
#include "sound.h"
#include "threed.h"
#include "swat.h"
#include "random.h"
#include "options.h"
#include "stars.h"
#include "missions.h"


DATAFILE *datafile;
BITMAP *buffer;


volatile int frame_count;
int speed_cap;

int old_cross_x, old_cross_y;

int draw_lasers;
int mcount;
int message_count;
char message_string[80];
int rolling;
int climbing;

int find_input;
char find_name[20];

/*
 * Read a line from a .cfg file.
 * Ignore blanks, comments and strip white space.
 */

void read_cfg_line (char *str, int max_size, FILE *fp)
{
	char *s;

	do
	{	
		fgets (str, max_size, fp);

		for (s = str; *s; s++)					/* End of line at LF or # */
		{
			if ((*s == '\n') || (*s == '#'))
			{
				*s = '\0';
				break;
			} 		
		}

		if (s != str)							/* Trim any trailing white space */
		{
			s--;
			while (isspace(*s))
			{
				*s = '\0';
				if (s == str)
					break;
				s--;
			}
		}

	} while (*str == '\0');
}


/*
 * Read in the scanner .cfg file.
 */

void read_scanner_config_file (char *filename)
{
	FILE *fp;
	char str[256];
	
	fp = fopen (filename, "r");
	if (fp == NULL)
		return;

	read_cfg_line (str, sizeof(str), fp);
	strcpy (scanner_filename, str);

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d,%d", &scanner_cx, &scanner_cy);
	scanner_cy += 385;

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d,%d", &compass_centre_x, &compass_centre_y);
	compass_centre_y += 385;
	
	fclose (fp);
}

/*
 * Read in the newkind.cfg file.
 */

void read_config_file (void)
{
	FILE *fp;
	char str[256];
	
	fp = fopen ("newkind.cfg", "r");
	if (fp == NULL)
		return;

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d", &speed_cap);

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d", &wireframe);

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d", &anti_alias_gfx);

	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d", &planet_render_style);
	
	read_cfg_line (str, sizeof(str), fp);
	sscanf (str, "%d", &hoopy_casinos);

	read_cfg_line (str, sizeof(str), fp);
	read_scanner_config_file (str);
		
	fclose (fp);
}






void frame_timer (void)
{
	frame_count++;
}
END_OF_FUNCTION(frame_timer);




/*
 * Initialise the game parameters.
 */

void initialise_game(void)
{
	set_rand_seed (time(NULL));
	current_screen = SCR_INTRO_ONE;

	restore_saved_commander();

	flight_speed = 1;
	flight_roll = 0;
	flight_climb = 0;
	docked = 1;
	front_shield = 255;
	aft_shield = 255;
	energy = 255;
	draw_lasers = 0;
	mcount = 0;
	hyper_ready = 0;
	detonate_bomb = 0;
	find_input = 0;
	
	create_new_stars();
	clear_universe();
	
	cross_x = -1;
	cross_y = -1;

	
	myship.max_speed = 40;		/* 0.27 Light Mach */
	myship.max_roll = 31;
	myship.max_climb = 8;		/* CF 8 */
	myship.max_fuel = 70;		/* 7.0 Light Years */
}


void finish_game (void)
{
	finish = 1;
	game_over = 1;
}




/*
 * Blit the back buffer to the screen.
 * This is done 12.5 times per second.
 */

void update_screen (void)
{
	while (frame_count < 1)
		rest (10);
	frame_count = 0;
	
	acquire_screen();
 	blit (buffer, screen, GFX_X_OFFSET, GFX_Y_OFFSET, GFX_X_OFFSET, GFX_Y_OFFSET, 513, 512);
	release_screen();
}



/*
 * Move the planet chart cross hairs to specified position.
 */


void move_cross (BITMAP *buf, int dx, int dy)
{
	if (current_screen == SCR_SHORT_RANGE)
	{
		cross_x += (dx * 4);
		cross_y += (dy * 4);
		return;
	}

	if (current_screen == SCR_GALACTIC_CHART)
	{
		cross_x += (dx * 2);
		cross_y += (dy * 2);
	}
}


/*
 * Draw the cross hairs at the specified position.
 */

void draw_cross (BITMAP *buf, int cx, int cy)
{
	if (current_screen == SCR_SHORT_RANGE)
	{
		xor_mode (TRUE);
		gfx_draw_colour_line (buf, cx - 16, cy, cx + 16, cy, GFX_COL_RED);
		gfx_draw_colour_line (buf, cx, cy - 16, cx, cy + 16, GFX_COL_RED);
		xor_mode (FALSE);
		return;
	}
	
	if (current_screen == SCR_GALACTIC_CHART)
	{
		xor_mode (TRUE);
		gfx_draw_colour_line (buf, cx - 8, cy, cx + 8, cy, GFX_COL_RED);
		gfx_draw_colour_line (buf, cx, cy - 8, cx, cy + 8, GFX_COL_RED);
		xor_mode (FALSE);
	}
}



void draw_laser_sights (void *hdc)
{
	int laser = 0;
	int x1,y1,x2,y2;
	
	switch (current_screen)
	{
		case SCR_FRONT_VIEW:
			gfx_display_centre_text (hdc, 32, "Front View", 120, GFX_COL_WHITE);
			laser = cmdr.front_laser;
			break;
		
		case SCR_REAR_VIEW:
			gfx_display_centre_text (hdc, 32, "Rear View", 120, GFX_COL_WHITE);
			laser = cmdr.rear_laser;
			break;

		case SCR_LEFT_VIEW:
			gfx_display_centre_text (hdc, 32, "Left View", 120, GFX_COL_WHITE);
			laser = cmdr.left_laser;
			break;

		case SCR_RIGHT_VIEW:
			gfx_display_centre_text (hdc, 32, "Right View", 120, GFX_COL_WHITE);
			laser = cmdr.right_laser;
			break;
	}
	

	if (laser)
	{
		x1 = 128 * GFX_SCALE;
		y1 = (96-8) * GFX_SCALE;
		y2 = (96-16) * GFX_SCALE;
   
		gfx_draw_colour_line (hdc, x1-1, y1, x1-1, y2, GFX_COL_GREY_1); 
		gfx_draw_colour_line (hdc, x1, y1, x1, y2, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, x1+1, y1, x1+1, y2, GFX_COL_GREY_1); 

		y1 = (96+8) * GFX_SCALE;
		y2 = (96+16) * GFX_SCALE;
		
		gfx_draw_colour_line (hdc, x1-1, y1, x1-1, y2, GFX_COL_GREY_1); 
		gfx_draw_colour_line (hdc, x1, y1, x1, y2, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, x1+1, y1, x1+1, y2, GFX_COL_GREY_1); 

		x1 = (128-8) * GFX_SCALE;
		y1 = 96 * GFX_SCALE;
		x2 = (128-16) * GFX_SCALE;
		   
		gfx_draw_colour_line (hdc, x1, y1-1, x2, y1-1, GFX_COL_GREY_1); 
		gfx_draw_colour_line (hdc, x1, y1, x2, y1, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, x1, y1+1, x2, y1+1, GFX_COL_GREY_1); 

		x1 = (128+8) * GFX_SCALE;
		x2 = (128+16) * GFX_SCALE;

		gfx_draw_colour_line (hdc, x1, y1-1, x2, y1-1, GFX_COL_GREY_1); 
		gfx_draw_colour_line (hdc, x1, y1, x2, y1, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, x1, y1+1, x2, y1+1, GFX_COL_GREY_1); 
	}
}


void arrow_right (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_MARKET_PRICES:
			buy_stock (hdc);
			break;
		
		case SCR_SHORT_RANGE:
		case SCR_GALACTIC_CHART:
			move_cross (hdc, 1, 0);
			break;

		case SCR_FRONT_VIEW:
		case SCR_REAR_VIEW:
		case SCR_RIGHT_VIEW:
		case SCR_LEFT_VIEW:
			if (flight_roll > 0)
				flight_roll = 0;
			else
			{
				decrease_flight_roll();
				decrease_flight_roll();
				rolling = 1;
			}
			break;
	}
}


void arrow_left (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_MARKET_PRICES:
			sell_stock (hdc);
			break;
		
		case SCR_SHORT_RANGE:
		case SCR_GALACTIC_CHART:
			move_cross (hdc, -1, 0);
			break;

		case SCR_FRONT_VIEW:
		case SCR_REAR_VIEW:
		case SCR_RIGHT_VIEW:
		case SCR_LEFT_VIEW:
			if (flight_roll < 0)
				flight_roll = 0;
			else
			{
				increase_flight_roll();
				increase_flight_roll();
				rolling = 1;
			}
			break;
	}
}


void arrow_up (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_MARKET_PRICES:
			select_previous_stock (hdc);
			break;

		case SCR_EQUIP_SHIP:
			select_previous_equip (hdc);
			break;

		case SCR_OPTIONS:
			select_previous_option (hdc);
			break;
		
		case SCR_SHORT_RANGE:
		case SCR_GALACTIC_CHART:
			move_cross (hdc, 0, -1);
			break;

		case SCR_FRONT_VIEW:
		case SCR_REAR_VIEW:
		case SCR_RIGHT_VIEW:
		case SCR_LEFT_VIEW:
			if (flight_climb > 0)
				flight_climb = 0;
			else
			{
				decrease_flight_climb();
			}
			climbing = 1;
			break;
	}
}



void arrow_down (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_MARKET_PRICES:
			select_next_stock (hdc);
			break;

		case SCR_EQUIP_SHIP:
			select_next_equip (hdc);
			break;
		
		case SCR_OPTIONS:
			select_next_option (hdc);
			break;
		
		case SCR_SHORT_RANGE:
		case SCR_GALACTIC_CHART:
			move_cross (hdc, 0, 1);
			break;

		case SCR_FRONT_VIEW:
		case SCR_REAR_VIEW:
		case SCR_RIGHT_VIEW:
		case SCR_LEFT_VIEW:
			if (flight_climb < 0)
				flight_climb = 0;
			else
			{
				increase_flight_climb();
			}
			climbing = 1;
			break;

	}
}


void return_pressed (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_EQUIP_SHIP:
			buy_equip (hdc);
			break;
		
		case SCR_OPTIONS:
			do_option (hdc);
			break;
	}	
}


void y_pressed (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_QUIT:
			finish_game();
			break;
	}
}


void n_pressed (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_QUIT:
			if (docked)
				display_commander_status (hdc);
			else
				current_screen = SCR_FRONT_VIEW;
			break;
	}
}


void d_pressed (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_GALACTIC_CHART:
		case SCR_SHORT_RANGE:
    		show_distance_to_planet (hdc);
			break;
	}
}


void f_pressed (HDC hdc)
{
	if ((current_screen == SCR_GALACTIC_CHART) ||
		(current_screen == SCR_SHORT_RANGE))
	{
		find_input = 1;
		*find_name = '\0';
		gfx_clear_text_area (hdc);
		gfx_display_text (hdc, 16, 340, "Planet Name?");
	}
}


void add_find_char (HDC hdc, int letter)
{
	char str[40];
	
	if (strlen (find_name) == 16)
		return;
		
	str[0] = toupper (letter);
	str[1] = '\0';
	strcat (find_name, str);

	sprintf (str, "Planet Name? %s", find_name);		
	gfx_clear_text_area (hdc);
	gfx_display_text (hdc, 16, 340, str);
}


void delete_find_char (HDC hdc)
{
	char str[40];
	int len;

	len = strlen (find_name);
	if (len == 0)
		return;
		
	find_name[len - 1] = '\0';	
		
	sprintf (str, "Planet Name? %s", find_name);		
	gfx_clear_text_area (hdc);
	gfx_display_text (hdc, 16, 340, str);
}

void o_pressed (HDC hdc)
{
	switch (current_screen)
	{
		case SCR_GALACTIC_CHART:
		case SCR_SHORT_RANGE:
    		move_cursor_to_origin (hdc);
			break;
	}
}



void handle_flight_keys (void *hdc)
{
	int keynum;
	int keycode, keyasc;
//	Matrix rotmat;
	

	poll_keyboard();

	if (key[KEY_F1])
	{
		find_input = 0;
		
		if (docked)
			launch_player();
		else
			current_screen = SCR_FRONT_VIEW;
	}

	if (key[KEY_F2])
	{
		find_input = 0;
		
		if (!docked)
			current_screen = SCR_REAR_VIEW;
	}

	if (key[KEY_F3])
	{
		find_input = 0;
		
		if (!docked)
			current_screen = SCR_LEFT_VIEW;
	}

	if (key[KEY_F4])
	{
		find_input = 0;
		
		if (!docked)
			current_screen = SCR_RIGHT_VIEW;
	}

	if (key[KEY_F4])
	{
		find_input = 0;
		
		if (docked)
			equip_ship (hdc);
	}
	
	if (key [KEY_F5])
	{
		find_input = 0;
		old_cross_x = -1;
		display_galactic_chart (hdc);
	}

	if (key [KEY_F6])
	{
		find_input = 0;
		old_cross_x = -1;
		display_short_range_chart (hdc);
	}

	if (key [KEY_F7])
	{
		find_input = 0;
		display_data_on_planet (hdc);
	}

	if (key[KEY_F8])
	{
		find_input = 0;
		display_market_prices (hdc);
	}	

	if (key[KEY_F9])
	{
		find_input = 0;
		display_commander_status (hdc);
	}

	if (key[KEY_F10])
	{
		find_input = 0;
		display_inventory (hdc);
	}
	
	if (key[KEY_F11])
	{
		find_input = 0;
		display_options (hdc);
	}

	if (find_input)
	{
		keynum = readkey();
		keycode = keynum >> 8;
		keyasc = keynum & 255;
		
		if (isalpha(keyasc))
		{
			add_find_char (hdc, keyasc);
			return;
		}
		
		if (keycode == KEY_ENTER)
		{
			find_input = 0;
			find_planet_by_name (hdc, find_name);
			return;
		}

		if (keycode == KEY_BACKSPACE)
		{
			delete_find_char (hdc);
			return;
		}
	
		return;		
	}
	
	if (key[KEY_Y])
		y_pressed (hdc);

	if (key[KEY_N])
		n_pressed (hdc);
/*
	if (key[KEY_W])
	{
		if (!docked)
		{
			set_init_matrix (rotmat);
			if (ship_count [SHIP_ASTEROID] == 0)
				add_new_ship (SHIP_ASTEROID, 0, 0, 3000, rotmat, 0, 0);
		}
	}
*/	
	if (key[KEY_A])
	{
		if (!docked)
			draw_lasers = fire_laser();
	}

	if (key[KEY_C])
	{
		if (!docked && cmdr.docking_computer)
			engage_docking_computer (hdc);
	}

	if (key[KEY_D])
		d_pressed (hdc);
	
	if (key[KEY_E])
	{
		if (!docked && cmdr.ecm)
			activate_ecm(1);
	}

	if (key[KEY_F])
		f_pressed (hdc);
	
	if ((key[KEY_H]) && (!docked))
	{
		if ((key[KEY_LCONTROL]) || (key[KEY_RCONTROL]))
			start_galactic_hyperspace();
		else
			start_hyperspace();
	}

	if (key[KEY_J])
	{
		if (!docked)
			jump_warp();
	}
	
	if (key[KEY_M])
	{
		if (!docked)
			fire_missile();
	}

	if (key[KEY_O])
		o_pressed (hdc);

	
	if (key[KEY_T])
	{
		if (!docked)
			arm_missile();		
	}

	if (key[KEY_U])
	{
		if (!docked)
			unarm_missile();
	}
	
	if (key[KEY_SPACE])
	{
		if (!docked)
		{
			if (flight_speed < myship.max_speed)
				flight_speed++;
		}
	}

	if (key[KEY_SLASH])
	{
		if (!docked)
		{
			if (flight_speed > 1)
				flight_speed--;
		}
	}

	if (key[KEY_S] || key[KEY_UP])
		arrow_up (hdc);
	
	if (key[KEY_X] || key[KEY_DOWN])
		arrow_down (hdc);

	if (key[KEY_COMMA] || key[KEY_LEFT])
		arrow_left (hdc);
		
	if (key[KEY_STOP] || key[KEY_RIGHT])
		arrow_right (hdc);
	
	if (key[KEY_ENTER])
		return_pressed (hdc);

	if (key[KEY_TAB])
	{
		if ((!docked) && (cmdr.energy_bomb))
		{
			detonate_bomb = 1;
			cmdr.energy_bomb = 0;
		}
	}		

	if (key[KEY_ESC])
	{
		if ((!docked) && (cmdr.escape_pod))
			abandon_ship (hdc);
	}
	
	
/*
	poll_joystick();	

	if (joy_up)
		arrow_up (hdc);
	
	if (joy_down)
		arrow_down (hdc);
	
	if (joy_left)
		arrow_left (hdc);
	
	if (joy_right)
		arrow_right (hdc);
*/
}



int checksum (unsigned char *block)
{
	int acc,carry;
	int i;

	acc = 0x49;
	carry = 0;
	for (i = 0x49; i > 0; i--)
	{
		acc += block[i-1] + carry;
		carry = acc >> 8;
		acc &= 255;
		acc ^= block[i];
	}

	return acc;
}


int save_commander_file (char *path)
{
	FILE *fp;
	unsigned char block[256];
	int i;
	int chk;
	
	fp = fopen (path, "wb");
	if (fp == NULL)
		return 1;
	
	block[0]  = cmdr.mission;
	block[1]  = docked_planet.d;
	block[2]  = docked_planet.b;
	block[3]  = cmdr.galaxy.a;
	block[4]  = cmdr.galaxy.b;
	block[5]  = cmdr.galaxy.c;
	block[6]  = cmdr.galaxy.d;
	block[7]  = cmdr.galaxy.e;
	block[8]  = cmdr.galaxy.f;
	block[9]  = (cmdr.credits >> 24) & 255;
	block[10] = (cmdr.credits >> 16) & 255;
	block[11] = (cmdr.credits >> 8) & 255;
	block[12] = cmdr.credits & 255;
	block[13] = cmdr.fuel;
	block[14] = 4;
	block[15] = cmdr.galaxy_number;
	block[16] = cmdr.front_laser;
	block[17] = cmdr.rear_laser;
	block[18] = cmdr.left_laser;
	block[19] = cmdr.right_laser;
	block[20] = 0;
	block[21] = 0;
	block[22] = cmdr.cargo_capacity + 2;

	for (i = 0; i < NO_OF_STOCK_ITEMS; i++)
		block[23+i] = cmdr.current_cargo[i];
	
	block[40] = cmdr.ecm ? 255 : 0;
	block[41] = cmdr.fuel_scoop ? 255 : 0;
	block[42] = cmdr.energy_bomb ? 0x7F : 0;
	block[43] = cmdr.energy_unit;
	block[44] = cmdr.docking_computer ? 255 : 0;
	block[45] = cmdr.galactic_hyperdrive ? 255 : 0;
	block[46] = cmdr.escape_pod ? 255 : 0;
	block[47] = 0;
	block[48] = 0;
	block[49] = 0;
	block[50] = 0;
	block[51] = cmdr.missiles;
	block[52] = cmdr.legal_status;
	
	for (i = 0; i < NO_OF_STOCK_ITEMS; i++)
		block[53+i] = stock_market[i].current_quantity;
	
	block[70] = cmdr.market_rnd;
	block[71] = cmdr.score & 255;
	block[72] = cmdr.score >> 8;
	block[73] = 0x20;

	chk = checksum (block);
	
	block[74] = chk ^ 0xA9;
	block[75] = chk;
	
	for (i = 76; i < 256; i++)
		block[i] = 0;

	if (fwrite (block, 256, 1, fp) != 1)
		return 1;
		
	if (fclose (fp) == EOF)
		return 1;	

	return 0;
}

int load_commander_file (char *path)
{
	FILE *fp;
	unsigned char block[256];
	int i;
	int chk;
	
	fp = fopen (path, "rb");
	if (fp == NULL)
		return 1;

	if (fread (block, 256, 1, fp) != 1)
		return 1;

	chk = checksum (block);

	if ((block[74] != (chk ^ 0xA9)) || (block[75] != chk))
		return 1;
	
	saved_cmdr.mission = block[0];

	saved_cmdr.ship_x = block[1];
	saved_cmdr.ship_y = block[2];
	
	saved_cmdr.galaxy.a = block[3];
	saved_cmdr.galaxy.b = block[4];
	saved_cmdr.galaxy.c = block[5];
	saved_cmdr.galaxy.d = block[6];
	saved_cmdr.galaxy.e = block[7];
	saved_cmdr.galaxy.f = block[8];;
	
	saved_cmdr.credits = block[9] << 24;
	saved_cmdr.credits += block[10] << 16;
	saved_cmdr.credits += block[11] << 8;
	saved_cmdr.credits += block[12];

	saved_cmdr.fuel = block[13];

	saved_cmdr.galaxy_number = block[15];
	saved_cmdr.front_laser = block[16];
	saved_cmdr.rear_laser = block[17];
	saved_cmdr.left_laser = block[18];
	saved_cmdr.right_laser = block[19];

	saved_cmdr.cargo_capacity = block[22] - 2;

	for (i = 0; i < NO_OF_STOCK_ITEMS; i++)
		saved_cmdr.current_cargo[i] = block[23+i];
	
	saved_cmdr.ecm = block[40];
	saved_cmdr.fuel_scoop = block[41];
	saved_cmdr.energy_bomb = block[42];
	saved_cmdr.energy_unit = block[43];
	saved_cmdr.docking_computer = block[44];
	saved_cmdr.galactic_hyperdrive = block[45];
	saved_cmdr.escape_pod = block[46];
	saved_cmdr.missiles = block[51];
	saved_cmdr.legal_status = block[52];
	
	for (i = 0; i < NO_OF_STOCK_ITEMS; i++)
		saved_cmdr.station_stock[i] = block[53+i];
	
	saved_cmdr.market_rnd = block[70];

	saved_cmdr.score = block[71];
	saved_cmdr.score += block[72] << 8;

	if (fclose (fp) == EOF)
		return 1;	

	return 0;
}



void set_commander_name (char *path)
{
	char *fname, *cname;
	int i;
	
	fname = get_filename (path);
	cname = cmdr.name;

	for (i = 0; i < 31; i++)
	{
		if (!isalnum(*fname))
			break;
		
		*cname++ = toupper(*fname++);
	}	

	*cname = '\0';
}

void save_commander_screen (HDC hdc)
{
	char path[255];
	int okay;
	int rv;
	
	current_screen = SCR_SAVE_CMDR;

	gfx_clear_display (hdc);
	gfx_display_centre_text (hdc, 10, "SAVE COMMANDER", 140, GFX_COL_GOLD);
	gfx_draw_line (hdc, 0, 36, 511, 36);
	update_screen();
	
	strcpy (path, cmdr.name);
	strcat (path, ".nkc");
	
	show_mouse (screen);
	okay = file_select ("Save Commander", path, "nkc");
	show_mouse (NULL);

	if (!okay)
	{
		display_options (hdc);
		return;
	}

	rv = save_commander_file (path);

	if (rv)
	{
		gfx_display_centre_text (hdc, 175, "Error Saving Commander!", 140, GFX_COL_GOLD);
		return;
	}
	
	gfx_display_centre_text (hdc, 175, "Commander Saved.", 140, GFX_COL_GOLD);

	set_commander_name (path);
	saved_cmdr = cmdr;
	saved_cmdr.ship_x = docked_planet.d;
	saved_cmdr.ship_y = docked_planet.b;
}


void load_commander_screen (HDC hdc)
{
	char path[255];
	int rv;

	gfx_clear_display (hdc);
	gfx_display_centre_text (hdc, 10, "LOAD COMMANDER", 140, GFX_COL_GOLD);
	gfx_draw_line (hdc, 0, 36, 511, 36);
	update_screen();
	
	
	strcpy (path, "jameson.nkc");
	
	show_mouse (screen);
	rv = file_select ("Load Commander", path, "nkc");
	show_mouse (NULL);

	if (rv == 0)
		return;

	rv = load_commander_file (path);

	if (rv)
	{
		saved_cmdr = cmdr;
		gfx_display_centre_text (hdc, 175, "Error Loading Commander!", 140, GFX_COL_GOLD);
		gfx_display_centre_text (hdc, 200, "Press any key to continue.", 140, GFX_COL_GOLD);
		update_screen();
		readkey();
		return;
	}
	
	restore_saved_commander();
	set_commander_name (path);
}



void run_first_intro_screen (void)
{
	snd_play_midi (SND_ELITE_THEME, TRUE);

	initialise_intro1();

	for (;;)
	{
		update_intro1 (buffer);

		update_screen();

 		poll_keyboard();

		if (key[KEY_Y])
		{
			snd_stop_midi();	
			load_commander_screen (buffer);
			break;
		}
		
		if (key[KEY_N])
		{ 
			snd_stop_midi();	
			break;
		}
	} 

}



void run_second_intro_screen (void)
{
	snd_play_midi (SND_BLUE_DANUBE, TRUE);
		
	initialise_intro2();

	flight_speed = 3;
	flight_roll = 0;
	flight_climb = 0;

	for (;;)
	{
		update_intro2 (buffer);

		update_screen();

		poll_keyboard();

		if (key[KEY_SPACE]) 
			break;

	} 

	snd_stop_midi();
}



/*
 * Draw the game over sequence. 
 * Just do a simple one for now.
 */

void run_game_over_screen (HDC hdc)
{
	int i;

	flight_speed = 10;
	flight_roll = 0;
	flight_climb = 0;
	
	for (i = 0; i < 100; i++)
	{
		gfx_clear_display (hdc);
		update_starfield (hdc);
		gfx_display_centre_text (hdc, 190, "GAME OVER", 140, GFX_COL_GOLD);
		update_screen();
	}
}




/*
 * Draw a break pattern (for launching, docking and hyperspacing).
 * Just draw a very simple one for the moment.
 */

void display_break_pattern (HDC hdc)
{
	int i;

	gfx_set_clip_region (buffer, 1, 1, 510, 383);
	gfx_clear_display (hdc);
	
	for (i = 0; i < 20; i++)
	{
		gfx_draw_circle (hdc, 256, 192, 30 + i * 15, GFX_COL_WHITE);
		update_screen();
	}	


	if (docked)
	{
		check_mission_brief (hdc);
		display_commander_status (hdc);
		update_console (hdc);
	}
	else
		current_screen = SCR_FRONT_VIEW;
}


void info_message (char *message)
{
	strcpy (message_string, message);
	message_count = 37;
//	snd_play_sample (SND_BEEP);
}




int main(int argc, char *argv[])
{
#ifdef __BEOS__ 
  /* set cwd to path of exe */ 
  /* to allow running from icon */ 
   
  int i; 
  char datapath[256]; 
  for (i=strlen(argv[0]) ; argv[0][i]!='/' ; i--); 
  strncpy(datapath, argv[0], i); 
  chdir(datapath); 
#endif 

	allegro_init();
	install_keyboard(); 
	install_timer();
	install_mouse();
//	load_joystick_data (NULL);


#ifdef ALLEGRO_WINDOWS	
 	if (set_gfx_mode(GFX_DIRECTX, 800, 600, 0, 0) != 0)
		set_gfx_mode(GFX_SAFE, 800, 600, 0, 0);
#else
	set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0);
#endif
		

	read_config_file();

	datafile = load_datafile("elite.dat");
	if (!datafile)
	{
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
      	allegro_message("Error loading %s!\n", "elite.dat");
      	return 1;
	}


	/* Create the screen buffer bitmap */
	buffer = create_bitmap (SCREEN_W, SCREEN_H);

	if (gfx_graphics_startup (buffer) == 1)
	{
		return 1;
	}
	
	/* Start the sound system... */
	snd_sound_startup();

	/* Install a timer to regulate the speed of the game... */

	LOCK_VARIABLE(frame_count);
	LOCK_FUNCTION(frame_timer);
	frame_count = 0;
	install_int (frame_timer, speed_cap);

	finish = 0;
	
	while (!finish)
	{
		game_over = 0;	
		initialise_game();
		dock_player (buffer);

		update_console(buffer);

		current_screen = SCR_FRONT_VIEW;
		run_first_intro_screen();
		run_second_intro_screen();

		old_cross_x = -1;
		old_cross_y = -1;

		dock_player (buffer);
		display_commander_status (buffer);
		
		while (!game_over)
		{
			snd_update_sound();
			update_screen();
			gfx_set_clip_region (buffer, 1, 1, 510, 383);

			if (message_count > 0)
				message_count--;

			rolling = 0;
			climbing = 0;

			if (keypressed())
				handle_flight_keys (buffer);

			if ((current_screen != SCR_FRONT_VIEW) && (current_screen != SCR_REAR_VIEW) &&
				(current_screen != SCR_LEFT_VIEW) && (current_screen != SCR_RIGHT_VIEW))
			{
				while (keypressed())
					readkey();
			}

			if (!rolling)
			{
				if (flight_roll > 0)
					decrease_flight_roll();
			
				if (flight_roll < 0)
					increase_flight_roll();
			}

			if (!climbing)
			{
				if (flight_climb > 0)
					decrease_flight_climb();

				if (flight_climb < 0)
					increase_flight_climb();
			}


			if (!docked)
			{		
				if ((current_screen == SCR_FRONT_VIEW) || (current_screen == SCR_REAR_VIEW) ||
					(current_screen == SCR_LEFT_VIEW) || (current_screen == SCR_RIGHT_VIEW))
				{
//					gfx_set_clip_region (buffer, 1, 1, 510, 383);
					gfx_clear_display (buffer);
					update_starfield (buffer);
				}

				update_universe (buffer);

				if (docked)
				{
					update_console (buffer);
					continue;
				}
					
				if ((current_screen == SCR_FRONT_VIEW) || (current_screen == SCR_REAR_VIEW) ||
					(current_screen == SCR_LEFT_VIEW) || (current_screen == SCR_RIGHT_VIEW))
				{
					if (draw_lasers)
					{
						draw_laser_lines(buffer);
						draw_lasers--;
					}
					
					draw_laser_sights (buffer);
				}

				if (message_count > 0)
					gfx_display_centre_text (buffer, 358, message_string, 120, GFX_COL_WHITE);
					
				if (hyper_ready)
				{
					display_hyper_status (buffer);
					if ((mcount & 3) == 0)
					{
						countdown_hyperspace();
					}
				}

//				gfx_set_clip_region (buffer, 0, 0, 512, 512);
			
				mcount--;
				if (mcount < 0)
					mcount = 255;

				if ((mcount & 7) == 0)
					regenerate_shields();

				if ((mcount & 31) == 10)
					update_altitude();
				
				if ((mcount & 31) == 20)
					update_cabin_temp();
					
				if (mcount == 0)
					random_encounter();
					
				cool_laser();				
				time_ecm();

				update_console (buffer);
				
			}

			if (current_screen == SCR_BREAK_PATTERN)
				display_break_pattern (buffer);
			
			if ((cross_x != old_cross_x) ||
				(cross_y != old_cross_y))
			{
				if (old_cross_x != -1)
					draw_cross (buffer, old_cross_x, old_cross_y);

				old_cross_x = cross_x;
				old_cross_y = cross_y;

				draw_cross (buffer, old_cross_x, old_cross_y);
			}
		}

		if (!finish)		
			run_game_over_screen(buffer);
	}

	snd_sound_shutdown();
	
	gfx_graphics_shutdown (buffer);
	
	destroy_bitmap(buffer);

	unload_datafile(datafile);

	return 0;
}

END_OF_MAIN();
