/*
 * Options.c
 */

#include <stdlib.h>
#include <string.h>

#include "elite.h"
#include "config.h"
#include "gfx.h"
#include "options.h"
#include "main.h"
#include "docked.h" 

static int hilite_item;
 
#define NUM_OPTIONS 3

struct option
{
	char *text;
	int docked_only;
};

static struct option option_list[NUM_OPTIONS] =
{
	{"Save Commander",	1},
	{"Load Commander",	1},
	{"Quit",			0}	
};





void quit_screen (HDC hdc)
{
	current_screen = SCR_QUIT;

	gfx_clear_display (hdc);
	gfx_display_centre_text (hdc, 10, "GAME OPTIONS", 140, GFX_COL_GOLD);
	gfx_draw_line (hdc, 0, 36, 511, 36);

	gfx_display_centre_text (hdc, 175, "QUIT GAME (Y/N)?", 140, GFX_COL_GOLD);		
}


void display_option_item (HDC hdc, int i)
{
	int y;
	int col;
	
	y = (384 - (30 * NUM_OPTIONS)) / 2;
	y += i * 30;
	col = ((!docked) && option_list[i].docked_only) ? GFX_COL_GREY_1 : GFX_COL_WHITE;

	gfx_display_centre_text (hdc, y, option_list[i].text, 120, col);
}


void highlight_option (HDC hdc, int i)
{
	int y;

	
	if ((hilite_item != -1) && (hilite_item != i))
	{
		y = (384 - (30 * NUM_OPTIONS)) / 2;
		y += hilite_item * 30;
		gfx_clear_area (hdc, 2, y, 510, y + 15);
		display_option_item (hdc, hilite_item);		
	}

	y = (384 - (30 * NUM_OPTIONS)) / 2;
	y += i * 30;
	
	gfx_draw_rectangle (hdc, 2, y, 510, y + 15, GFX_COL_DARK_RED);
	display_option_item (hdc, i);		

	hilite_item = i;
}

void select_previous_option (HDC hdc)
{
	if (hilite_item > 0)
		highlight_option (hdc, hilite_item - 1);
}

void select_next_option (HDC hdc)
{
	if (hilite_item < (NUM_OPTIONS - 1))
		highlight_option (hdc, hilite_item + 1);
}


void do_option (HDC hdc)
{
	if ((!docked) && option_list[hilite_item].docked_only)
		return;

	switch (hilite_item)
	{
		case 0:
			save_commander_screen (hdc);
			break;
			
		case 1:
			load_commander_screen (hdc);
			display_commander_status (hdc);
			break;
		
		case 2:
			quit_screen (hdc);
			break;
	}
}


void display_options (HDC hdc)
{
	int i;

	current_screen = SCR_OPTIONS;
	
	gfx_clear_display (hdc);
	gfx_display_centre_text (hdc, 10, "GAME OPTIONS", 140, GFX_COL_GOLD);
	gfx_draw_line (hdc, 0, 36, 511, 36);
	gfx_display_centre_text (hdc, 320, "Version: Beta 2.1", 120, GFX_COL_WHITE);
	
	for (i = 0; i < NUM_OPTIONS; i++)
		display_option_item (hdc, i);

	hilite_item = -1;
	highlight_option (hdc, 0);
}
