/*
 * Elite - The New Kind.
 *
 * Reverse engineered from the BBC disk version of Elite.
 * Additional material by C.J.Pinder.
 *
 * The original Elite code is (C) I.Bell & D.Braben 1984.
 * This version re-engineered in C by C.J.Pinder 1999-2000.
 *
 * email: <christian@newkind.co.uk>
 *
 *
 */

/*
 * swat.c
 *
 * Special Weapons And Tactics.
 */

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "gfx.h"
#include "elite.h"
#include "vector.h"
#include "swat.h"
#include "shipdata.h"
#include "space.h"
#include "main.h"
#include "sound.h"
#include "random.h"
#include "trade.h"
 

int laser_counter;
int laser;
int laser2;
int laser_x;
int laser_y;

int ecm_active;
int missile_target;
int ecm_ours;
int in_battle;

void reset_weapons (void)
{
	laser_temp = 0;
	laser_counter = 0;
	laser = 0;
	ecm_active = 0;
	missile_target = MISSILE_UNARMED;
}

 
void launch_enemy (int un, int type, int flags, int bravery)
{
	int newship;
	struct univ_object *ns;
	
	newship = add_new_ship (type, universe[un].location.x, universe[un].location.y,
							universe[un].location.z, universe[un].rotmat,
							universe[un].rotx, universe[un].rotz);

	if (newship == -1)
	{
		return;
	}

	ns = &universe[newship];
	
	if ((universe[un].type == SHIP_CORIOLIS) || (universe[un].type == SHIP_DODEC))
	{
		ns->velocity = 32;
		ns->location.x += ns->rotmat[2].x * 2; 		
		ns->location.y += ns->rotmat[2].y * 2; 		
		ns->location.z += ns->rotmat[2].z * 2;
	}

	ns->flags = flags;
	ns->rotz /= 2;
	ns->rotz *= 2;
	ns->bravery = bravery;
}


int in_target (int type, double x, double y, double z)
{
	double size;
	
	if (z < 0)
		return 0;
		
	size = ship_list[type]->size;

	return ((x*x + y*y) <= size);	
}



void make_angry (int un)
{
	int type;
	
	type = universe[un].type;
	
	if ((type == SHIP_CORIOLIS) || (type == SHIP_DODEC))
	{
		universe[un].flags |= FLG_ANGRY;
		return;
	}
	
	universe[un].rotx = 4;
	universe[un].acceleration = 2;

	if (type > SHIP_ROCK)
		universe[un].flags |= FLG_ANGRY;
}


void explode_object (int un)
{

	cmdr.score++;

	al_trace ("explode_object: %d, %d\n", un, cmdr.score);
	
	if ((cmdr.score & 255) == 0)
		info_message ("Right On Commander!");
	
	snd_play_sample (SND_EXPLODE);
	universe[un].flags |= FLG_DEAD;

	if (universe[un].type == SHIP_CONSTRICTOR)
		cmdr.mission = 2;
}


void check_target (int un, struct univ_object *flip)
{
	int i;
	int cnt;
	struct univ_object *univ;
	
	univ = &universe[un];
	
	if (in_target (univ->type, flip->location.x, flip->location.y, flip->location.z))
	{
		if ((missile_target == MISSILE_ARMED) && (univ->type >= 0))
		{
			missile_target = un;
			info_message ("Target Locked");
//			snd_play_sample (SND_BEEP);
		}
	
		if (laser)
		{
			snd_play_sample (SND_HIT_ENEMY);

			if ((univ->type != SHIP_CORIOLIS) && (univ->type != SHIP_DODEC))
			{			
				if ((univ->type == SHIP_CONSTRICTOR) || (univ->type == SHIP_COUGAR))
				{
					if (laser == (MILITARY_LASER & 127))
						univ->energy -= laser / 4;
				}
				else
				{
					univ->energy -= laser;
				}
			}

			if (univ->energy <= 0)
			{
				explode_object (un);
				
				if (univ->type != SHIP_ASTEROID)
				{
					cnt = rand255();
					if (cnt < 128)
					{
						cnt &= ship_list[univ->type]->max_loot;
						for (i = 0; i < cnt; i++)
						{
							launch_enemy (un, SHIP_CARGO, 0,0);
						}
					}
				}
			}
					
			make_angry (un);
		}
	}
}


void check_missiles (int un)
{
	int i;
	
	if (missile_target == un)
	{
		missile_target = MISSILE_UNARMED;
		info_message ("Target Lost");
	}

	for (i = 0; i < MAX_UNIV_OBJECTS; i++)
	{
		if ((universe[i].type == SHIP_MISSILE) && (universe[i].target == un))
			universe[i].flags |= FLG_DEAD;
	}
}


void activate_ecm (int ours)
{
	if (ecm_active == 0)
	{
		ecm_active = 32;
		ecm_ours = ours;
		snd_play_sample (SND_ECM);
	}
}


void time_ecm (void)
{
	if (ecm_active != 0)
	{
		ecm_active--;
		if (ecm_ours)
			decrease_energy (-1);
	}
}


void arm_missile (void)
{
	if ((cmdr.missiles != 0) && (missile_target == MISSILE_UNARMED))
		missile_target = MISSILE_ARMED;
}


void unarm_missile (void)
{
	missile_target = MISSILE_UNARMED;
}

void fire_missile (void)
{
	int newship;
	struct univ_object *ns;
	Matrix rotmat;

	if (missile_target < 0)
		return;
	
	set_init_matrix (rotmat);
	rotmat[2].z = 1.0;
	rotmat[0].x = -1.0;
	
	newship = add_new_ship (SHIP_MISSILE, 0, -28, 14, rotmat, 0, 0);

	if (newship == -1)
	{
		info_message ("Missile Jammed");
		return;
	}

	ns = &universe[newship];
	
	ns->velocity = flight_speed * 2;
	ns->flags = FLG_ANGRY;
	ns->target = missile_target;

	if (universe[missile_target].type > SHIP_ROCK)
		universe[missile_target].flags |= FLG_ANGRY;
	
	cmdr.missiles--;
	missile_target = MISSILE_UNARMED;
	
	snd_play_sample (SND_MISSILE);
}



void track_player (HDC hdc, int un, Vector dirvec)
{	
	double direction;
	double dir;
	Vector nvec;
	int type;
	int rat;
	double rat2;
	
	rat = 3;
	rat2 = 0.0208;		/* 88.8 degrees (approx) */ 
	
	type = universe[un].type;
	
	nvec = unit_vector(&dirvec);
	direction = vector_dot_product (&nvec, &universe[un].rotmat[2]); 

	if ((type != SHIP_MISSILE) &&
		(universe[un].distance <= 4096) && (direction < -0.33) &&
		(ship_list[type]->laser_strength != 0))
	{
		universe[un].flags |= FLG_FIRING | FLG_HOSTILE;		
		if (direction < -0.364)
		{
			damage_ship (ship_list[type]->laser_strength, universe[un].location.z >= 0.0);
			universe[un].acceleration--;
			if (((universe[un].location.z >= 0.0) && (front_shield == 0)) ||
				((universe[un].location.z < 0.0) && (aft_shield == 0)))
				snd_play_sample (SND_INCOMMING_FIRE_2);
			else
				snd_play_sample (SND_INCOMMING_FIRE_1);
			return;
		}				
	} 

	if ((type == SHIP_MISSILE) ||
		(((fabs(universe[un].location.z) >= 0x300) ||
		  (fabs(universe[un].location.x) > 512) ||
		  (fabs(universe[un].location.y) > 512)) &&
		 ((rand255() & 127) < universe[un].bravery)))
	{
		nvec.x = -nvec.x;
		nvec.y = -nvec.y;
		nvec.z = -nvec.z;
		direction = -direction;
	}

	dir = vector_dot_product (&nvec, &universe[un].rotmat[1]);
	if (fabs(dir) > rat2)
		universe[un].rotx = (dir < 0) ? rat : -rat;
		
	if (abs(universe[un].rotz) < 16)
	{
		dir = vector_dot_product (&nvec, &universe[un].rotmat[0]);
		if (fabs(dir) > rat2)
		{
			universe[un].rotz = (dir < 0) ? rat : -rat;

			if (universe[un].rotx < 0)
				universe[un].rotz = -universe[un].rotz;
		}		
	}


	if (direction >= 0.223)
	{
		universe[un].acceleration = 3;
		return;
	}

	if ((direction >= 0.1875) || (direction <= -0.1875))
		universe[un].acceleration = (type == SHIP_MISSILE) ? -2 : -1;
}



void missile_tactics (HDC hdc, int un)
{
	struct univ_object *missile;
	struct univ_object *target;
	Vector vec;

	missile = &universe[un];
	
	if (ecm_active)
	{
		snd_play_sample (SND_EXPLODE);
		missile->flags |= FLG_DEAD;		
		return;
	}

	if (missile->target == 0)
	{
		if (missile->distance < 256)
		{
			missile->flags |= FLG_DEAD;
			snd_play_sample (SND_EXPLODE);
			damage_ship (250, missile->location.z >= 0.0);
			return;
		}

		track_player (hdc, un, missile->location);
		return;
	}

	target = &universe[missile->target];

	vec.x = missile->location.x - target->location.x;
	vec.y = missile->location.y - target->location.y;
	vec.z = missile->location.z - target->location.z;
	
	if ((fabs(vec.x) < 256) && (fabs(vec.y) < 256) && (fabs(vec.z) < 256))
	{
		missile->flags |= FLG_DEAD;		

		if ((target->type != SHIP_CORIOLIS) && (target->type != SHIP_DODEC))
			explode_object (missile->target);
		else
			snd_play_sample (SND_EXPLODE);

		return;
	}	

	
	if ((rand255() < 16) && (target->flags & FLG_HAS_ECM))
	{
		activate_ecm (0);
		return;
	}

	track_player (hdc, un, vec);
}


void tactics (HDC hdc, int un)
{
	int type;
	int energy;
	int maxeng;
	
	type = universe[un].type;

	if ((type == SHIP_PLANET) || (type == SHIP_SUN))
		return;
	
	if (universe[un].flags & FLG_DEAD)
		return;
	
	if (type == SHIP_MISSILE)
	{
		if (universe[un].flags & FLG_ANGRY)
			missile_tactics (hdc, un);
		return;
	}

	if (((un ^ mcount) & 7) != 0)
		return;

	if (type == SHIP_ESCAPE_CAPSULE)
	{
//		sps1();
//		ta15();
		return;
	}
	

	if ((type == SHIP_CORIOLIS) || (type == SHIP_DODEC))
	{
 		if (universe[un].flags & FLG_ANGRY) 
		{
			if ((rand() & 255) < 240)
				return;
		
			if (ship_count[SHIP_VIPER] >= 7)
				return; 

			launch_enemy (un, SHIP_VIPER, FLG_ANGRY | FLG_HAS_ECM, 113);
			return;
		}

		/* Do docking and launching code... */
		return;
	}

	if ((universe[un].flags & FLG_ANGRY) == 0)
		return; 
	
	if ((type == SHIP_THARGLET) && (ship_count[SHIP_THARGOID] == 0))
	{
		universe[un].flags = 0;
		universe[un].velocity /= 2;
		return;
	}

	if (universe[un].energy < ship_list[type]->energy)
		universe[un].energy++;

	if (rand255() > 250)
		universe[un].rotz = rand255() | 0x68;
	
	maxeng = ship_list[type]->energy;
	energy = universe[un].energy;

	if (energy < (maxeng / 2))
	{
		if ((energy < (maxeng / 8)) && (rand255() > 230) && (type != SHIP_THARGOID))
		{
//			enemy_escape_capsule();
			return;				
		}

		if ((universe[un].missiles != 0) && (ecm_active == 0) &&
			(universe[un].missiles > (rand255() & 31)))
		{
			universe[un].missiles--;
			if (type == SHIP_THARGOID)
				launch_enemy (un, SHIP_THARGLET, FLG_ANGRY, universe[un].bravery);
			else
			{
				launch_enemy (un, SHIP_MISSILE, FLG_ANGRY, 126);
				info_message ("INCOMING MISSILE");
			}
			return;
		}
	}

	track_player (hdc, un, universe[un].location);
}

void draw_laser_lines (HDC hdc)
{
	if (wireframe)
	{
		gfx_draw_colour_line (hdc, 32 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, 48 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, 208 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y, GFX_COL_WHITE);
		gfx_draw_colour_line (hdc, 224 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y, GFX_COL_WHITE);
	}
	else
	{
		gfx_draw_triangle (hdc,  32 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y,  48 * GFX_SCALE, GFX_VIEW_BY, GFX_COL_RED);
		gfx_draw_triangle (hdc, 208 * GFX_SCALE, GFX_VIEW_BY, laser_x, laser_y, 224 * GFX_SCALE, GFX_VIEW_BY, GFX_COL_RED);
	}		 
}


int fire_laser (void)
{
	if ((laser_counter == 0) && (laser_temp < 242))
	{
		switch (current_screen)
		{
			case SCR_FRONT_VIEW:
				laser = cmdr.front_laser;
				break;
			
			case SCR_REAR_VIEW:
				laser = cmdr.rear_laser;
				break;
					
			case SCR_RIGHT_VIEW:
				laser = cmdr.right_laser;
				break;
					
			case SCR_LEFT_VIEW:
				laser = cmdr.left_laser;
				break;
				
			default:
				laser = 0;
		}

		if (laser != 0)
		{
			laser_counter = (laser > 127) ? 0 : (laser & 0xFA);
			laser &= 127;
			laser2 = laser;

			snd_play_sample (SND_PULSE);
			laser_temp += 8;
			if (energy > 1)
				energy--;
			
			laser_x = ((rand() & 3) + 128 - 2) * GFX_SCALE;
			laser_y = ((rand() & 3) + 96 - 2) * GFX_SCALE;
			
			return 2;
		}
	}

	return 0;
}


void cool_laser (void)
{
	laser = 0;

	if (laser_temp > 0)
		laser_temp--;
					
	if (laser_counter > 0)
		laser_counter--;
				
	if (laser_counter > 0)
		laser_counter--;
}


void create_trader (void)
{
	int x,y,z;
	int rz;
	int newship;
	Matrix rotmat;
	int rnd;
	int type;

//	z = 9728;
//	x = rand255();
//	y = rand255();

	z = 12000;
	x = 1000 + (randint() & 8191);
	y = 1000 + (randint() & 8191);

	if (rand255() > 127)
		x = -x;
	if (rand255() > 127)
		y = -y;
//	x *= 4;
	
	rz = rand255() / 2;
	set_init_matrix (rotmat);
	rotmat[2].z = -1.0;

	type = SHIP_COBRA3 + (rand255() & 3);
		
	newship = add_new_ship (type, x, y, z, rotmat, 0, rz);
	if (newship != -1)
	{
		rnd = rand255();
		universe[newship].velocity = (rnd & 31) | 16;
		universe[newship].bravery = rnd / 2;

		if (rnd & 1)
			universe[newship].flags |= FLG_HAS_ECM;

		if (rnd & 2)
			universe[newship].flags |= FLG_ANGRY; 
	}
}



/* Check for a random asteroid encounter... */

void check_for_asteroids (void)
{
	int x,y,z;
	int rz, rx;
	int newship;
	Matrix rotmat;

	if ((rand255() >= 35) || (ship_count[SHIP_ASTEROID] >= 3))
		return;

//	z = 9728;
//	x = rand255();

	z = 12000;
	x = 1000 + (randint() & 8191);
	y = 1000 + (randint() & 8191);

	if (rand255() > 127)
		x = -x;
	if (rand255() > 127)
		y = -y;

//	x *= 4;
	
//	rx = (rand255() - 128) | 0x6F;
	rx = 16;
	rz = rand255() > 127 ? -127 : 127;
	set_init_matrix (rotmat);
		
	newship = add_new_ship (SHIP_ASTEROID, x, y, z, rotmat, rx, rz);
	if (newship != -1)
	{
		universe[newship].velocity = (rand255() & 31) | 16; 
	}
}


/* If we've been a bad boy then send the cops after us... */

void check_for_cops (void)
{
	int x,y,z;
	int newship;
	Matrix rotmat;
	int offense;

	offense = carrying_contraband() * 2;
	if (ship_count[SHIP_VIPER] == 0)
		offense |= cmdr.legal_status;

	if (rand255() >= offense)
		return;

	set_init_matrix (rotmat);

//	z = 8192;
//	x = 8192;
//	y = 8192;

	z = 12000;
	x = 1000 + (randint() & 8191);
	y = 1000 + (randint() & 8191);

	if (rand255() > 127)
		x = -x;
	if (rand255() > 127)
		y = -y;

	newship = add_new_ship (SHIP_VIPER, x, y, z, rotmat, 0, 0);
	if (newship != -1)
	{
		universe[newship].flags = FLG_ANGRY;
		if (rand255() > 245)
			universe[newship].flags |= FLG_HAS_ECM;
		
		universe[newship].bravery = ((rand255() * 2) | 64) & 127;  
	}
}


void check_for_others (void)
{
	int x,y,z;
	int newship;
	Matrix rotmat;
	int gov;
	int rnd;
	int type;
	int i;

	gov = current_planet_data.government; 
	rnd = rand255();

	if ((gov != 0) && ((rnd >= 90) || ((rnd & 7) < gov)))
		return;	

	if (rand255() < 100)
	{
		set_init_matrix (rotmat);

//		z = 8192;
//		x = 8192;
//		y = 8192;

		z = 12000;
		x = 1000 + (randint() & 8191);
		y = 1000 + (randint() & 8191);

		if (rand255() > 127)
			x = -x;
		if (rand255() > 127)
			y = -y;

		rnd = rand255();

		if ((cmdr.mission == 1) && (cmdr.galaxy_number == 1) &&
			(docked_planet.d == 144) && (docked_planet.b == 33) &&
			(ship_count[SHIP_CONSTRICTOR] == 0))
		{
			type = SHIP_CONSTRICTOR;
		}
		else
			type = SHIP_COBRA3_LONE + (rnd & 3) + (rnd > 127);
		
		newship = add_new_ship (type, x, y, z, rotmat, 0, 0);
		if (newship != -1)
		{
			universe[newship].flags = FLG_ANGRY;
			if ((rand255() > 200) || (type == SHIP_CONSTRICTOR))
				universe[newship].flags |= FLG_HAS_ECM;
		
			universe[newship].bravery = ((rand255() * 2) | 64) & 127;
			in_battle = 1;  
		}	
		
		return;
	}	

	/* Pack hunters... */
	
	set_init_matrix (rotmat);

//	z = 8192;
//	x = 8192;
//	y = 8192;

		z = 12000;
		x = 1000 + (randint() & 8191);
		y = 1000 + (randint() & 8191);

	if (rand255() > 127)
		x = -x;
	if (rand255() > 127)
		y = -y;

	rnd = rand255() & 3;
	
	for (i = 0; i <= rnd; i++)
	{
		type = SHIP_SIDEWINDER + (rand255() & rand255() & 7);
		newship = add_new_ship (type, x, y, z, rotmat, 0, 0);
		if (newship != -1)
		{
			universe[newship].flags = FLG_ANGRY;
			if (rand255() > 245)
				universe[newship].flags |= FLG_HAS_ECM;
		
			universe[newship].bravery = ((rand255() * 2) | 64) & 127;
			in_battle++;  
		}
	}
	
}


void random_encounter (void)
{
	if ((ship_count[SHIP_CORIOLIS] != 0) || (ship_count[SHIP_DODEC] != 0))
		return;

	if ((rand255() & 7) == 0)
	{
		create_trader();
		return;
	}
		
	check_for_asteroids();

	check_for_cops();	

	if (ship_count[SHIP_VIPER] != 0)
		return;

	if (in_battle)
		return;
		
	check_for_others();	
}


void abandon_ship (HDC hdc)
{
	int i;

	cmdr.escape_pod = 0;
	cmdr.legal_status = 0;
	cmdr.fuel = myship.max_fuel;
	
	for (i = 0; i < NO_OF_STOCK_ITEMS; i++)
		cmdr.current_cargo[i] = 0;
	
	snd_play_sample (SND_DOCK);					
	dock_player (hdc);
	current_screen = SCR_BREAK_PATTERN;
}

